"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const Rx = __importStar(require("rxjs"));
const operators_1 = require("rxjs/operators");
const commander_1 = __importDefault(require("commander"));
const types_1 = require("./types");
const radar_relay_1 = require("./exchanges/radar-relay");
const kraken_1 = require("./exchanges/kraken");
const kyber_1 = require("./exchanges/kyber");
const exchanges_1 = require("./exchanges");
const debug = require('debug')('exchange-aggregator');
exports.exchangeOrderObservableCreators = {
    [types_1.Exchange.RADAR_RELAY]: (options) => radar_relay_1.standardizeStream(radar_relay_1.getObservableRadarRelayOrders(options)),
    [types_1.Exchange.KRAKEN]: (options) => kraken_1.standardizeStream(kraken_1.getObservableKrakenOrders(options)),
    [types_1.Exchange.KYBER]: (options) => kyber_1.standardizeStream(kyber_1.getObservableKyberOrders(options)),
};
exports.createExchangeOrderObservable = R.curry((options, exchange) => {
    return exports.exchangeOrderObservableCreators[exchange](options);
});
exports.createExchangeOrderObservables = R.curry((options, exchanges = [types_1.Exchange.RADAR_RELAY]) => {
    return exchanges.map(exports.createExchangeOrderObservable(options));
});
commander_1.default.description('The exchange-aggregator testing tool.');
commander_1.default
    .command('orders [exchanges...]')
    .option('-n, --network <network>', 'The network (KOVAN/MAINNET).', /(MAINNET|KOVAN)/, 'MAINNET')
    .option('-b, --base <symbol>', 'The base token symbol.', /[A-Z]{3,4}/, 'ZRX')
    .option('-q, --quote <symbol>', 'The quote token symbol.', /[A-Z]{3,4}/, 'ETH')
    .description('Retrieve orders from the given exchanges')
    .action((args, options) => {
    const supported = Object.keys(types_1.Exchange);
    const invalid = R.difference(args, supported);
    if (invalid && invalid.length) {
        console.error(`Invalid exchanges %s. Supported exchanges are %s.`, invalid.join(', '), supported.join(', '));
        process.exit(1);
    }
    const exchanges = args.length ? args : supported;
    const opts = {
        base: options.base,
        quote: options.quote,
        network: types_1.Network[options.network],
    };
    debug('Aggregating orderbook for %s.', exchanges.join(', '));
    const observables = exports.createExchangeOrderObservables(opts, exchanges);
    Rx.merge(...observables)
        .pipe(operators_1.tap(value => exchanges_1.debugEvent(value)))
        .subscribe();
    process.stdin.resume();
});
commander_1.default.parse(process.argv);
