"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Rx = __importStar(require("rxjs"));
const axios_1 = __importDefault(require("axios"));
const operators_1 = require("rxjs/operators");
const types_1 = require("../../types");
const debug = require('debug')('exchange-aggregator:kraken');
const getHttpUrl = (base, quote, network) => {
    switch (network) {
        case types_1.Network.MAINNET:
            return `https://api.kraken.com/0/public/Depth?pair=${base}${quote}`;
        default:
            throw new Error('Kraken only supports the MAINNET network.');
    }
};
exports.getObservableKrakenOrders = ({ base, quote, network, }) => {
    const polling$ = Rx.interval(5000).pipe(operators_1.switchMap(() => {
        debug(`Loading snapshot for market ${base}-${quote}.`);
        const url = getHttpUrl(base, quote, network);
        return Rx.from(axios_1.default.get(url).then(result => result.data));
    }), operators_1.map(data => {
        if (data.error && data.error.length) {
            throw new Error('Error while trying to fetch the snapshot.');
        }
        const orderbook = (data.result && data.result[`X${base}X${quote}`]) || {
            asks: [],
            bids: [],
        };
        return Object.assign({}, orderbook, { base,
            quote });
    }), operators_1.tap(orderbook => {
        debug('Loaded orderbook with %s bids and %s asks.', orderbook.bids.length, orderbook.asks.length);
    }), operators_1.retryWhen(error => {
        debug('Kraken failed with error.');
        return error.pipe(operators_1.delay(10000));
    }), operators_1.distinctUntilChanged());
    return polling$.pipe(operators_1.share());
};
const standardizeOrder = (order) => ({
    price: parseFloat(order[0]),
    volume: parseFloat(order[1]),
});
exports.standardizeStream = (stream$) => {
    return stream$.pipe(operators_1.map(orderbook => {
        const asks = orderbook.asks.map(order => standardizeOrder(order));
        const bids = orderbook.bids.map(order => standardizeOrder(order));
        return {
            event: types_1.StandardizedMessageType.SNAPSHOT,
            exchange: types_1.Exchange.KRAKEN,
            base: orderbook.base,
            quote: orderbook.quote,
            asks,
            bids,
        };
    }));
};
